function [Snijpunt,IndexRoosterpunt,Lengte]=Rivier(rivier,nx,ny,X0,Y0)
% [Snijpunt,IndexRoosterpunt,Lengte]=Rivier(rivier,nx,ny,X0,Y0)
%
%  Bekijk celletjes  ter grootte hx=X0/(nx+1) bij hy=Y0/(ny+1)
% rond de roosterpunten  (x(i,j),y(i,j)), met x(i,j)=i*hx, y(i,j)=j*hy. 
% IndexRoosterpunt is 2 bij l-1 waarbij l-1 het aantal celletjes is dat de 
% rivier doorsnijdt.
% Als (i;j)=IndexRoosterpunt(:,k), dan is (i;j) de (i,j)-coordinaat van het k-de celletje
% dat de rivier doorsnijdt (met k=1,2,...,l-1).
% De rivier komt het k-de celletje binnen in punt (x1;y1)=Snijpunt(:,k)
% en verlaat dit k-de celletje op het punt (x2;y2)=Snijpunt(:,k+1).
% Lengte(k) is de lengte van de rivier in celletje k
  
global SHOWGRAPHS


hx=X0/(nx+1); hy=Y0/(ny+1); 
%% genereer de x en y coordinaten voor roosterlijnen
%% in "half" waarden
xi=[0,(1.5:1:nx)*hx,X0]; yj=[0,(1.5:1:ny)*hy,Y0];

%% de rivier volgt de lijn R = {[x;y] | alpha*x+beta*y=gamma}
alpha=rivier(1); beta=rivier(2); gamma=rivier(3);

hx=X0/(nx+1); hy=Y0/(ny+1); 
%% genereer de x en y coordinaten voor roosterlijnen
%% in "half" waarden
xh=[0,(1.5:1:nx)*hx,X0]; yh=[0,(1.5:1:ny)*hy,Y0];

%% R is ook de lijn   {s+r*t | t reeel getal}
%% vind het startunt s en de richtingsvector r
  r=[-beta;alpha];
  if (abs(alpha)>abs(beta))
    s=[gamma/alpha;0]; ;
  else
    s=[0;gamma/beta];
  end
  %% vind de snijpunten van deze lijn R met de roosterlijnen 
  %% dwz, vind de t waarvoor s(1)+r(1)*t=xi en s(2)+r(2)*t=yj;
  %% Snijpunt=s*ones(size(t))+r*t;
  Snijpunt=track(r,s,xh,yh);

  %% vind de index  (heel waardige)  van de roosterpunten in de buurt
  %% van de lijnstukjes tussen twee op een volgende snijpunten
  %%  (i,j)=IndexRoosterpunt(:,k), Roosterpunt=H*IndexRoosterpunt
  l=size(Snijpunt,2); J=1:l-1; H=diag([hx,hy]);
  IndexRoosterpunt=round(H\(Snijpunt(:,J)+Snijpunt(:,J+1))/2);
  %% associeer randroosterpunten met inwendige roosterpunten
  j=find(IndexRoosterpunt(1,:)==0);    IndexRoosterpunt(1,j)=1;
  j=find(IndexRoosterpunt(1,:)==nx+1); IndexRoosterpunt(1,j)=nx;
  j=find(IndexRoosterpunt(2,:)==0);    IndexRoosterpunt(2,j)=1;
  j=find(IndexRoosterpunt(2,:)==ny+1); IndexRoosterpunt(2,j)=ny;

  %% Lengte(i) is de lengte tussen snijpunt(i) en snijpunt(i+1)
  %% en hoort bij hets roosterpunt(i)
  Lengte=Snijpunt(:,J+1)-Snijpunt(:,J);
  Lengte=Lengte.*Lengte; Lengte=Lengte(1,:)+Lengte(2,:); Lengte=sqrt(Lengte);
  
%% Grafische uitvoer om te controleren of de celletjes
%% die de rivier doorkruist ook daadwerkelijk gevonden worden
if 0 % SHOWGRAPHS
  plotrivier(Snijpunt,H*IndexRoosterpunt,X0,Y0,xh,yh)
end

return
%==================================================================
function Snijpunt=track(r,s,x,y)
%Snijpunt=track(r,s,x,y)
%   vind de snijpunten van de lijn s+r*t (s en r kolomvectoren)
%   en de horizontale en verticalen lijnen met respectivelijke
%   verticale waarde y(j) en horizontale waarde x(i)
%

  
  %% vind de snijpunten van deze lijn met de roosterlijnen
  %% dwz, vind de t waarvoor s(1)+t*r(1)=x en s(2)+t*r(2)=y;
  %% t heeft length length(x)+length(y)
  %% check of de lijn s+r*t bijna horizontaal of bijna verticaal loopt
  t=[];  
  if (abs(r(1))>eps*abs(r(2))), t=[t,(x-s(1))/r(1)]; end
  if (abs(r(2))>eps*abs(r(1))), t=[t,(y-s(2))/r(2)]; end
  %% sorteer t, verwijder t-waarden die ongeveer gelijk zijn 
  %% aan eerdere t-waarden
  e=10^8; tt=round(t*e)/e; [tt,j]=unique(tt); t=t(j);
  Snijpunt=s*ones(size(t))+r*t;
  %% selecteer alleen die snijpunten die in het gebied vallen
  J=find(Snijpunt(1,:)<=x(end) & x(1)<=Snijpunt(1,:) & ...
  Snijpunt(2,:)<=y(end) & y(1)<=Snijpunt(2,:));
  Snijpunt=Snijpunt(:,J);


return
%==================================================================
function plotrivier(SnijPunten,RoosterPunt,X0,Y0,xx,yy)
%%   Grafische uitvoer om te controleren of de celletjes
%%   die de rivier doorkruist ook daadwerkelijk gevonden worden
%%   Plot het "half" rooster , de stukjes rivier tussen "half" roosterlijnen,
%%   de (heel) roosterpunten waaraan het stukje rivier wordt toe gekend.

  BYSTEP=1;
  II=[0*yy,xx;X0+0*yy,xx]; JJ=[yy,0*xx;yy,Y0+0*xx];

  [k,l]=size(RoosterPunt);
  BYSTEP = (max(k,l)<20 & BYSTEP);
  if BYSTEP
  ii=[0,1]; 
  hold off
  for jj=1:l
     plot(SnijPunten(1,ii+jj),SnijPunten(2,ii+jj))
     axis([0,X0,0,Y0]), hold on
     plot(II,JJ,':')
     plot(RoosterPunt(1,jj),RoosterPunt(2,jj),'o')
     pause, hold off
  end
  end
  
  if (max(k,l)<40)
    h1=plot(II,JJ,'m:'); axis([0,X0,0,Y0]), hold on
    h3=plot(SnijPunten(1,:),SnijPunten(2,:),'*');
    h2=plot(SnijPunten(1,:),SnijPunten(2,:),'-','LineWidth',2);
    h4=plot(RoosterPunt(1,:),RoosterPunt(2,:),'go');
    h=[h1(1,1);h2;h3;h4];
    title('Rivier en roosterpunten waaraan de rivierwaarde wordt toegekend')
    legend(h,'half rooster','rivier','snijpunten rivier-rooster','roosterpunten')
    pause
    hold off
  end
  
return
