function [x,Rho,SUCCESS]=GCR(A,b,x,kmax,tol)
%[x,Rho,SUCCESS]=GCR(A,b,x0,kmax,tol) 
%
%  A is a matrix, b the right hand side vector
%  x0 is the initial guess
%
%  x is the solution of Ax=b such that |b-Ax|<tol.|r0|
%  solved with GCR 
%  and kmax is the maximum number of iteration steps;
%  Here r0=b-A*x0, the initial residual.
%  
% Rho is 2 by m, where m is the required number of iteration steps
% Rho(1,j) the residual reduction  norm(rk,2)/norm(b,2) 
% in step j, after application of Rho(2,j) MVs
%
% SUCCESS=1 if the required residual reduction tol has been reached in
% less than kmax steps




if kmax==0, x=b; return, end

% find the dimension of the problem
n=size(b,1); 
ShowMethod('GCR');


U=zeros([n(1),kmax]); C=U; Sigma=zeros(1,kmax); 

% compute initial residual


tol0=tol;
tol=tol*tol*(b'*b); 


r=b-A*x; 

k=0;
rho=r'*r;
Rho=rho; K=k;
while k<kmax

   if  rho<tol, break, end
 
%%%%%%%%%%%%% ????

   rho=r'*r;    
   Rho=[Rho,rho]; K=[K,k]; 
   ShowIntermediateSolution(x,k);
 
end
% the first row of Rho is the row of norm(r_k,2):
%          (norm(r_0,2),norm(r_1,2),...,norm(r_k,2),...,nomr(r_kmax,2))
% the second row of Rho is  row of indices k :
%          K=[0,1,2,3, ..., k,..,kmax]
Rho=[sqrt(Rho);K];
SUCCESS=(k<kmax);

ShowConvergenceHist(K,Rho(1,:),SUCCESS,tol0,'GCR')

return
