function [x,Rho,SUCCESS]=PCG(A,b,x,kmax,tol,Msolve)
% [x,Rho,SUCCESS]=PCG(A,b,x,kmax,tol,D,ptype) 
%
%  A is a function subroutine, with b=A(x),  it computes b=A*x
% Msolve  is a function subroutine, with u=Msolve(r),  it solves M*u=r for u
%
%  x is the solution of Ax=b such that |b-Ax|<tol.|r0|
%  solved with precondition CG (conjugate gradients)
%  and kmax is the maximum number of iteration steps;
%  The derivation of the algorithm relies on the assumption that
%  A and M are symmetric
%
% Rho is 2 by m, where m is the required number of iteration steps
% Rho(1,j) the residual reduction  norm(rk,2)/norm(r0,2) 
% in step j, after application of Rho(2,j) MVs
%
% SUCCESS=1 if the required residual reduction tol has been reached in
% less than kmax steps


PRECOND=(nargin>5);

% find the dimension of the problem
n=size(b,1); 
ShowMethod('CG');

tol0=tol;
tol=tol0*tol0*(b'*b);
r=b-feval(A,x); 

Rho=zeros([1,kmax]); K=Rho;

k=0; u=0*b; 
rho=r'*r;
while k<kmax

   ro=r'*r;  
   k=k+1; Rho(k)=ro; K(k)=k-1;
   if  ro<tol, break, end

   if PRECOND, c=feval(Msolve,r); else, c=r; end
   rho1=rho; rho=r'*c; beta=rho/rho1;
   u=c+u*beta;  
   c=feval(A,u);
   sigma=u'*c; alpha=rho/sigma;
   x=x+u*alpha;
   r=r-c*alpha;
  
   ShowIntermediateSolution(x,k);
 
end

K=K([1:k]); Rho=[sqrt(Rho([1:k]));K];
SUCCESS=(k<kmax);

ShowConvergenceHist(K,Rho(1,:),SUCCESS,tol0,'PCG')

return
