install


%% "profile on"  and "profile viewer" (last line of main)  gives an
%%  analysis of the time spend on each line of the code
% profile on

global SHOWGRAPHS SHOWINTERMEDIATESOLUTION
%% select SHOWGRAPHS=1 to view convergence histories 
SHOWGRAPHS=1;
%% select SHOWINTERMEDIATESOLUTION to view intermediate approximate solutions
SHOWINTERMEDIATESOLUTION=0;

%%_________________________________________________________________________
%%  selectie probleem, discretisatie parameters
Model='Model14';
% nx=1200; ny=1200; 
nx=400; ny=350; 
% nx=80; ny=120; 

%%_________________________________________________________________________
%% MODEL INLEZEN
  
%% Geef model gegevens en de waarde van de discretisatieparameters
%% weer op het scherm
Tekst_PDE(Model,nx,ny);
%% Geef een grafische voorstelling van de model gegevens  (in Figure 1)
%% rivieren(blauwe lijnen), putten (blauwe punten), 
%% diffusie coefficienten (rode lijnen), stromings veld (groene pijlen)
Domein(Nivos,Schaal);

%_________________________________________________________________________
% DISCRETISATIE, opstellen stencil
   t0=clock;
[alphac,alphaw,alphao,alphaz,alphan,f]=Discretiseer(Model,nx,ny);
   disp(['discretisatietijd = ',num2str(etime(clock,t0))])
%%_________________________________________________________________________
%% MATRIX EN RECHTERLID OPSTELLEN
   t0=clock;
[A,b]=Vectoriseer(alphac,alphaw,alphao,alphan,alphaz,f);
   disp(['vectorisatietijd  = ',num2str(etime(clock,t0))])
   
   
if nx*ny<3000
%% compute and display the eigenvalues of A
%% Is feasible only if the dimension is not to large
   Lambda=eig(full(A));
   figure
   plot(real(Lambda),imag(Lambda),'*')
   title('Spectrum of A')
end

%%_________________________________________________________________________
%% OPLOSSEN STELSEL

if 0
%% Los op met de standaard MATLAB oplos routines  ("MATLAB backslash)
      t0=clock; 
      fprintf('\nGaussian elimination (MATLAB)\n')
   X=A\b;
      t1=etime(clock,t0);
      re=b-A*X;
      str='#MVs=%d, #steps=%d, |r| = %8.2e, t = %g\n';
      fprintf(str,0,0,sqrt(re'*re),t1);
%% Zet de oplossingsvector X om naar een roosterfunctie phi
%% (van de grondwaterdruk/concentratie verontreiniging) en bepaal
%% het bijbehorend stromingsveld (U,V)=-(a d_x phi , b d_y phi) met
%% gebruikmaken van de randvoorwaarden die in het model gedefinieerd zijn
      [phi,U,V]=Continueer(X,nx,ny,Model);
%% Plot de oplossing phi (grondwaterdruk/concentratie verontreiniging)
%% en het stromingsveld (U,V)
      Vertoon(Model,phi,U,V,Nivos,Schaal)
end

if 0
      t0=clock; 
      fprintf('\nGaussian elimination (MATLAB)\n')
   [L,U,p,q]=lu(A,'vector'); Y=U\(L\b(p)); X(q)=Y; 
      t1=etime(clock,t0);
      re=b-A*X;
      str='#MVs=%d, #steps=%d, |r| = %8.2e, t = %g\n'; 
      fprintf(str,0,0,sqrt(re'*re),t1);
      if nx*ny<10001, 
        figure,  spy(A(p,q),'b.'), hold on, 
        spy(L+U,'r.'); hold on, 
      end
end


%% Bouw de preconditionneerder
%     t0=clock; 
% D=RILU(alphac,alphaw,alphao,alphaz,alphan,0.97); 
%     disp(['bouwtijd preconditioneerder = ',num2str(etime(clock,t0))])

tol=10^(-12);
kmax=200;
x0=0*b;
   figure
      t0=clock; 

%% los op 
SOLVER='GCR';
switch SOLVER
 case 'GCR'
   [X,Rho,SUCCESS]=GCR(A,b,x0,kmax,tol);
 case 'PGCR'
   %% Preconditioneer GCR impliciet
   %%   For an explanation of the suggested implementation,
   %%   see the pdf "Incorporating preconditioners" 
   %%   on the website for this course
   PreProcess(A,b,x0,D,'Diagonal');
   [X,Rho,SUCCESS]=PGCR(A,b,x0,kmax,tol,'Msolve');
 case {'Flexible GCR','PLMR','PCG'}
   %% Flexible GCR allowing implicit as well as explicite preconditioning
   [preb,prex0]=PreProcess(A,b,x0);
   switch SOLVER
     case 'Flexible GCR'
          [X,Rho,SUCCESS]=FlexGCR('PMV',preb,prex0,kmax,tol,'Msolve');
     case 'PLMR'
          [X,Rho,SUCCESS]=PLMR('PMV',preb,prex0,kmax,tol,'Msolve');
     case 'PCG'
          [X,Rho,SUCCESS]=PCG('PMV',preb,prex0,kmax,tol,'Msolve');
   end
   X=PostProcess(X);
 otherwise
   X=A\b;
end

      t1=etime(clock,t0);
   
   % To check attained residual accuracy, compute true residual
   re=b-A*X;
   % display statistics (perfomance of the method) 
   str='#MVs=%d, #steps=%d, |r| = %8.2e, t = %g\n';
   fprintf(str,Rho(2,end),size(Rho,2),sqrt(re'*re),t1);

      disp(['Oplos tijd       = ',num2str(t1)])

%%_________________________________________________________________________
%% Terug naar rooster:, en BEREKEN het VECTORVELD (U,V):
%% Zet de oplossingsvector X om naar een roosterfunctie phi
%% (van de grondwaterdruk/concentratie verontreiniging) en bepaal
%% het bijbehorend stromingsveld (U,V)=-(a d_x phi , b d_y phi) met
%% gebruikmaking van de randvoorwaarden die in het model gedefinieerd zijn
 
   t0=clock; [phi,U,V]=Continueer(X,nx,ny,Model);
   disp(['cont.tijd        = ',num2str(etime(clock,t0))])

%%_________________________________________________________________________
%% PLOTTEN OPLOSSING
%% Plot de oplossing phi (grondwaterdruk/concentratie verontreiniging)
%% en het stromingsveld (U,V)

%% grafische uitvoer parameters
Nivos=15;  %% het aantal niveau krommen in de plot van de waterdruk 
Mesh=1;    %% voor een uitvoer in 3 d
Schaal=10; %% schaalt de lengte van de vectors in het stromingsveld

  Vertoon(Model,phi,U,V,Nivos,Schaal)

% profile viewer
